import json
import unittest

from lakey_finicity.responses import InstitutionsListResponse

EXAMPLE_INSTITUTIONS_RESPONSE = '''
{
"found":7843,
"displaying":25,
"moreAvailable":true,
"createdDate":1582938670,
"institutions":[
{
"id":15436,
"name":"Mass Mutual Financial Group(Retirement Access)",
"aha":false,
"accountTypeDescription":"Workplace Retirement",
"phone":"1-866-630-5295",
"urlHomeApp":"https://www.massmutual.com/",
"urlLogonApp":"https://retirementsolutions.financialtrans.com/tf/myPLAN/Welcome?cz=d07001719051417031511001318",
"oauthEnabled":false,
"urlForgotPassword":"",
"urlOnlineRegistration":"",
"class":"retirement",
"specialText":"Please enter your Mass Mutual Financial Group (Retirement Access) Email Address and Password required for login.",
"address":{
"city":"Atlanta",
"state":"GA",
"country":"USA",
"postalCode":"30357-2566",
"addressLine1":"Post Office Box 78566",
"addressLine2":""
},
"currency":"USD",
"email":"https://www.amsouthdailyelect.com/contact.asp",
"oauthInstitutionId":null
},
{
"id":15974,
"name":"Spectra Bank",
"aha":false,
"accountTypeDescription":"Banking",
"phone":"817-284-9598",
"urlHomeApp":"https://www.spectra.bank/",
"urlLogonApp":"https://www.myebanking.net/riverbendbank/login/login.aspx",
"oauthEnabled":false,
"urlForgotPassword":"",
"urlOnlineRegistration":"",
"class":"banking",
"specialText":"Please enter your River Bend Bank Access ID and Password required for login.",
"address":{
"city":"Texas",
"state":"IL",
"country":"USA",
"postalCode":"76181-0070",
"addressLine1":"P.O. Box 185070",
"addressLine2":"Fort Worth"
},
"currency":"USD",
"email":"feedback@riverbendbank.com",
"oauthInstitutionId":null
},
{
"id":1411,
"name":"Royal Bank of Canada",
"aha":true,
"accountTypeDescription":"Banking",
"phone":"1-800-769-2555",
"urlHomeApp":"http://www.rbcbank.com",
"urlLogonApp":"https://www1.royalbank.com/cgi-bin/rbaccess/rbunxcgi?F6=1&F7=IB&F21=IB&F22=IB&REQUEST=ClientSignin&LANGUAGE=ENGLISH",
"oauthEnabled":false,
"urlForgotPassword":"",
"urlOnlineRegistration":"",
"class":"banking",
"specialText":"Please enter your Royal Bank of Canada Username or Client Card Number and Password.",
"address":{
"city":"",
"state":"",
"country":"CAN",
"postalCode":"",
"addressLine1":"",
"addressLine2":""
},
"currency":"USD",
"email":"http://www.royalbank.com/rd_form1.html",
"oauthInstitutionId":null
},
{
"id":7999,
"name":"Old Navy Credit Card",
"aha":false,
"accountTypeDescription":"Credit Cards/Accounts",
"phone":"1-800-653-6289",
"urlHomeApp":"http://www.oldnavy.com",
"urlLogonApp":"https://lakey_finicity.rfecom.com/eSecurity/Login/login.action?clientId=oldnavy&accountType=generic&langId=en",
"oauthEnabled":false,
"urlForgotPassword":"",
"urlOnlineRegistration":"",
"class":"creditCards",
"specialText":"Please enter your Old Navy User Name and Password Required for Login",
"address":{
"city":"Grove City",
"state":"OH",
"country":"USA",
"postalCode":"43123-8605",
"addressLine1":"200 Old Navy Lane",
"addressLine2":""
},
"currency":"USD",
"email":"http://www.oldnavy.com/asp/feedback_other.asp?FromPg=Other",
"oauthInstitutionId":null
},
{
"id":14121,
"name":"Bank2",
"aha":false,
"accountTypeDescription":"Banking",
"phone":"1(405) 946-2265",
"urlHomeApp":"https://www.bank2.biz",
"urlLogonApp":"https://olb.bank2online.com/",
"oauthEnabled":false,
"urlForgotPassword":"",
"urlOnlineRegistration":"",
"class":"banking",
"specialText":"Please enter your Bank2 Username and password required for login.",
"address":{
"city":"Oklahoma City",
"state":"OK",
"country":"USA",
"postalCode":"73108",
"addressLine1":"909 S. Meridian",
"addressLine2":""
},
"currency":"USD",
"email":"",
"oauthInstitutionId":null
},
{
"id":12721,
"name":"Banco Popular - Web Cash Manager",
"aha":false,
"accountTypeDescription":"Banking",
"phone":"1-800-377-0800",
"urlHomeApp":"https://www.popularbank.com/#home",
"urlLogonApp":"https://webcmpr.bancopopular.com/K1/",
"oauthEnabled":false,
"urlForgotPassword":null,
"urlOnlineRegistration":null,
"class":"Banking",
"specialText":"Please enter your Banco Popular Customer ID, User ID and Password required for login.",
"address":{
"city":"New York",
"state":"NY",
"country":"USA",
"postalCode":"10004",
"addressLine1":"85 Broad Street",
"addressLine2":""
},
"currency":"USD",
"email":"popularnet@popular.com",
"oauthInstitutionId":null
},
{
"id":13037,
"name":"BMW Financial Services",
"aha":false,
"accountTypeDescription":"Loan",
"phone":"1-800-831-1117",
"urlHomeApp":"https://www.bmwusa.com/",
"urlLogonApp":"https://securelogin.bmwusa.com/login?returnUrl=%2F",
"oauthEnabled":false,
"urlForgotPassword":null,
"urlOnlineRegistration":null,
"class":"Loan",
"specialText":"Please Enter your BMW Financial Services Email Address and Password required for login.",
"address":{
"city":"Woodcliff Lake",
"state":"NJ",
"country":"USA",
"postalCode":"84109-0488",
"addressLine1":"BMW of North America, LLC",
"addressLine2":"300 Chestnut Ridge Roa"
},
"currency":"USD",
"email":null,
"oauthInstitutionId":null
},
{
"id":14341,
"name":"WageWorks Spending Account",
"aha":false,
"accountTypeDescription":"Investment",
"phone":"1 (877) 924-3967",
"urlHomeApp":"https://myspendingaccount.wageworks.com/Welcome/PortalLandingPage.aspx",
"urlLogonApp":"https://myspendingaccount.wageworks.com/Welcome/PortalLandingPage.aspx",
"oauthEnabled":false,
"urlForgotPassword":null,
"urlOnlineRegistration":null,
"class":"Investment",
"specialText":null,
"address":{
"city":"San Mateo",
"state":"CA",
"country":"USA",
"postalCode":"10016",
"addressLine1":"NA",
"addressLine2":"NA"
},
"currency":"USD",
"email":null,
"oauthInstitutionId":null
},
{
"id":15680,
"name":"Fifth Third Direct - Business Banking",
"aha":false,
"accountTypeDescription":null,
"phone":"1-866-671-5353",
"urlHomeApp":"https://www.53.com/wps/portal/personal",
"urlLogonApp":"https://express.53.com/portal/authn/#/login",
"oauthEnabled":false,
"urlForgotPassword":null,
"urlOnlineRegistration":null,
"class":"Business Banking",
"specialText":"Please enter your Fifth Third Direct - Business Banking User ID and Password required for login. \\t",
"address":{
"city":"Cincinnati",
"state":"OH",
"country":"USA",
"postalCode":"45263",
"addressLine1":"Fifth Third Center",
"addressLine2":"38 Fountain Square Plaza"
},
"currency":"USD",
"email":null,
"oauthInstitutionId":null
},
{
"id":16021,
"name":"Starbucks Card",
"aha":false,
"accountTypeDescription":"Credit Card",
"phone":"(800) 782-7282",
"urlHomeApp":"https://www.starbucks.com/",
"urlLogonApp":"https://app.starbucks.com/account/signin",
"oauthEnabled":false,
"urlForgotPassword":null,
"urlOnlineRegistration":null,
"class":"Credit Card",
"specialText":"Please enter your Starbucks Username/ email and Password required for login.",
"address":{
"city":"San Angelo",
"state":"TX",
"country":"USA",
"postalCode":"98124-3717",
"addressLine1":"P.O. Box 60650",
"addressLine2":"San Angelo"
},
"currency":"USD",
"email":"https://customerservice.starbucks.com",
"oauthInstitutionId":null
},
{
"id":100389,
"name":"HealthEquity",
"aha":false,
"accountTypeDescription":"Banking",
"phone":"877.694.3942",
"urlHomeApp":"https://www.healthequity.com",
"urlLogonApp":"https://www.healthequity.com/HE.aspx",
"oauthEnabled":false,
"urlForgotPassword":"",
"urlOnlineRegistration":"",
"class":"banking",
"specialText":"Please enter your HealthEquity Username or Email Address and Password required for login.",
"address":{
"city":"Draper ",
"state":"UT",
"country":"USA",
"postalCode":"84020",
"addressLine1":"15 W Scenic Pointe Dr",
"addressLine2":""
},
"currency":"USD",
"email":"http://www.healthequity.com",
"oauthInstitutionId":null
},
{
"id":170837,
"name":"Navirefi: Student Loan Refinancing",
"aha":false,
"accountTypeDescription":"Loan",
"phone":null,
"urlHomeApp":"https://navirefi.com/",
"urlLogonApp":"https://login2.navirefi.com/CALM2/pages/CALM2CookieJsCheck.jsp?sourceAppName=NRILP",
"oauthEnabled":false,
"urlForgotPassword":null,
"urlOnlineRegistration":null,
"class":"loan",
"specialText":"Please enter your Navirefi User ID and Password required for login.",
"address":{
"city":"Wilkes-Barre",
"state":"PA",
"country":"USA",
"postalCode":"18773-9115",
"addressLine1":"NaviRefi",
"addressLine2":"P.O. Box 9115"
},
"currency":"USD",
"email":null,
"oauthInstitutionId":null
},
{
"id":170838,
"name":"InBank",
"aha":false,
"accountTypeDescription":"Banking",
"phone":"303.805.0321",
"urlHomeApp":"https://inbank.com/",
"urlLogonApp":"https://inbank.com/personal-banking/",
"oauthEnabled":false,
"urlForgotPassword":null,
"urlOnlineRegistration":null,
"class":"banking",
"specialText":"Please enter your IN Bank's User ID and Password required for login.",
"address":{
"city":"Aurora",
"state":"CO",
"country":"USA",
"postalCode":"800165",
"addressLine1":"24450 e smoky hill rd",
"addressLine2":"Aurora-80016"
},
"currency":"USD",
"email":null,
"oauthInstitutionId":null
},
{
"id":170839,
"name":"City National Bank Credit Card Online",
"aha":false,
"accountTypeDescription":"Credit Card",
"phone":null,
"urlHomeApp":"https://cnb.cardmanager.com/",
"urlLogonApp":"https://cnb.cardmanager.com/",
"oauthEnabled":false,
"urlForgotPassword":null,
"urlOnlineRegistration":null,
"class":"Credit Card",
"specialText":"Please enter your User Name and Password required for login. \\t",
"address":{
"city":"Lost Angeles",
"state":"CA",
"country":"USA",
"postalCode":"90071",
"addressLine1":"City National Plaza",
"addressLine2":"555 South Flower Street"
},
"currency":"USD",
"email":null,
"oauthInstitutionId":null
},
{
"id":170840,
"name":"Community Bank & Trust of Florida (MIDFLORIDA CCU)",
"aha":false,
"accountTypeDescription":"Banking",
"phone":"352-369-1000",
"urlHomeApp":"https://online.cbtfl.com",
"urlLogonApp":"https://online.cbtfl.com/cbtfl/uux.aspx#/login",
"oauthEnabled":false,
"urlForgotPassword":null,
"urlOnlineRegistration":null,
"class":"banking",
"specialText":"Please enter your Login ID & Password required for login.",
"address":{
"city":"Ocala",
"state":"FL",
"country":"USA",
"postalCode":"34471",
"addressLine1":"1603 SW 19th Avenue",
"addressLine2":"Ocala"
},
"currency":"USD",
"email":null,
"oauthInstitutionId":null
},
{
"id":170846,
"name":"Gazelle",
"aha":false,
"accountTypeDescription":null,
"phone":null,
"urlHomeApp":"https://www.daveramsey.com/",
"urlLogonApp":"https://gazelle.id.ramseysolutions.net/sign-in?&client_id=debit-web-app&code_challenge_method=S256&code_challenge=OmLfNzomO5fdRbNFfH-MwOwAtLntlN_ENOCimI4dllw&redirect_uri=https%3A%2F%2Fwww.gazelleintense.com%2Fauth%2Fsession%2Faccess&response_type=code&sc",
"oauthEnabled":false,
"urlForgotPassword":null,
"urlOnlineRegistration":null,
"class":"1",
"specialText":"Please enter your Gazelle Intense Email and Password required for login.",
"address":{
"city":"Franklin",
"state":"TN",
"country":"USA",
"postalCode":"37064",
"addressLine1":"Ramsey Solutions",
"addressLine2":"1011 Reams Fleming Blvd"
},
"currency":"USD",
"email":null,
"oauthInstitutionId":null
},
{
"id":170848,
"name":"Prosperity Bank - Treasury Center",
"aha":false,
"accountTypeDescription":"Business Banking",
"phone":"1-855-888-2242",
"urlHomeApp":"https://www.prosperitybankusa.com",
"urlLogonApp":"https://prosperity.olbanking.com/",
"oauthEnabled":false,
"urlForgotPassword":null,
"urlOnlineRegistration":null,
"class":"Business Banking",
"specialText":"Please enter your Prosperity Bank - Treasury Center's Company ID, User ID and Password required for login.",
"address":{
"city":"El Campo",
"state":"TX",
"country":"USA",
"postalCode":"77437",
"addressLine1":"1301 North Mechanic",
"addressLine2":""
},
"currency":"USD",
"email":"treasurymanagement.support@prosperitybankusa.com",
"oauthInstitutionId":null
},
{
"id":170852,
"name":"Platinum Federal Credit Union",
"aha":false,
"accountTypeDescription":"Banking",
"phone":"404-297-9797",
"urlHomeApp":"https://www.platinumfcu.org/",
"urlLogonApp":"https://platinumfcu.onlinecu.com/?user",
"oauthEnabled":false,
"urlForgotPassword":null,
"urlOnlineRegistration":null,
"class":"Banking",
"specialText":"Please enter your Platinum Federal username and password required for login.",
"address":{
"city":"1059 Mistletoe Road",
"state":"GA",
"country":"USA",
"postalCode":"30033",
"addressLine1":"Decatur",
"addressLine2":"1000-A"
},
"currency":"USD",
"email":"memberservice@platinumfcu.org",
"oauthInstitutionId":null
},
{
"id":170853,
"name":"Key bank - Businesses & Institutions",
"aha":false,
"accountTypeDescription":"Business Banking",
"phone":null,
"urlHomeApp":"https://www.key.com/businesses-institutions/index.jsp",
"urlLogonApp":"https://keynavigator.key.com/ktt/cmd/logonFromKeyComNew",
"oauthEnabled":false,
"urlForgotPassword":null,
"urlOnlineRegistration":null,
"class":"Business Banking",
"specialText":"Please enter your Key Bank Business & Institutions User ID & Password required for login",
"address":{
"city":"Cleveland",
"state":"OH",
"country":"USA",
"postalCode":"44114",
"addressLine1":"127 Public Square",
"addressLine2":""
},
"currency":"USD",
"email":null,
"oauthInstitutionId":null
},
{
"id":170856,
"name":"U-Haul Federal Credit Union",
"aha":false,
"accountTypeDescription":"Banking",
"phone":"322172438",
"urlHomeApp":"https://uhaulfedcu.com/",
"urlLogonApp":"https://www.ea.financial-net.com/vbsts/Login/UserPassLogin.aspx?ReturnUrl=%2fvbsts%2f%3fwa%3dwsignin1.0%26wtrealm%3dhttps%253a%252f%252fwww.financial-net.com%252f%26wctx%3drm%253d0%2526id%253dpassive%2526ru%253d%25252fuhaulfedcu%25252fsecurity%25252fOlbCm",
"oauthEnabled":false,
"urlForgotPassword":null,
"urlOnlineRegistration":null,
"class":"Banking",
"specialText":"Please enter your U-Haul Federal Credit Union Username &b Password required for login.",
"address":{
"city":"Phoenix",
"state":"AZ",
"country":"USA",
"postalCode":"85004",
"addressLine1":"2700 N 3rd St Suite",
"addressLine2":"1000-A"
},
"currency":"USD",
"email":null,
"oauthInstitutionId":null
},
{
"id":31,
"name":"Wells Fargo",
"aha":true,
"accountTypeDescription":"Banking",
"phone":"1-800-956-4442",
"urlHomeApp":"https://www.wellsfargo.com",
"urlLogonApp":"https://connect.secure.wellsfargo.com/auth/login/present?origin=cob",
"oauthEnabled":false,
"urlForgotPassword":"",
"urlOnlineRegistration":"",
"class":"banking",
"specialText":"Please enter your Username or Social Security Number and your Password required to login.",
"address":{
"city":"San Francisco",
"state":"CA",
"country":"USA",
"postalCode":"94163",
"addressLine1":"",
"addressLine2":""
},
"currency":"USD",
"email":"https://www.wellsfargo.com/per/per_ask_us.jhtml",
"oauthInstitutionId":null
},
{
"id":14007,
"name":"Bank of America",
"aha":false,
"accountTypeDescription":"Banking",
"phone":"1-800-792-0808",
"urlHomeApp":"https://secure.bankofamerica.com/",
"urlLogonApp":"https://secure.bankofamerica.com/login/sign-in/signOnV2Screen.go",
"oauthEnabled":false,
"urlForgotPassword":"",
"urlOnlineRegistration":"",
"class":"banking",
"specialText":"Please enter your Bank of America Online ID(Minimum 6 character) and Passcode required for login. (Note: We do not support the BOA Safe passcode feature. Please disable it at your bank's website to successfully add your accounts)",
"address":{
"city":"Charlotte",
"state":"NC",
"country":"USA",
"postalCode":"28255",
"addressLine1":"307 S. MAIN",
"addressLine2":""
},
"currency":"USD",
"email":"https://www.bankofamerica.com/contact/",
"oauthInstitutionId":null
},
{
"id":5,
"name":"Chase",
"aha":true,
"accountTypeDescription":"Banking",
"phone":"1-800-242-7324",
"urlHomeApp":"https://secure01c.chase.com/",
"urlLogonApp":"https://secure01c.chase.com/web/auth/#/logon/logon/chaseOnline",
"oauthEnabled":false,
"urlForgotPassword":"",
"urlOnlineRegistration":"",
"class":"banking",
"specialText":"Please enter your Chase User ID and Password. ",
"address":{
"city":"New York",
"state":"NY",
"country":"USA",
"postalCode":"10017",
"addressLine1":"270 Park Avenue",
"addressLine2":"270 Park Avenue, New York"
},
"currency":"USD",
"email":"https://www.bankone.com/contactus/#personal",
"oauthInstitutionId":null
},
{
"id":1003,
"name":"Citibank",
"aha":true,
"accountTypeDescription":"Credit Cards/Accounts",
"phone":"1-800-950-5114",
"urlHomeApp":"https://online.citibank.com/US/Welcome.c",
"urlLogonApp":"https://online.citi.com/US/login.do",
"oauthEnabled":false,
"urlForgotPassword":"",
"urlOnlineRegistration":"",
"class":"creditCards",
"specialText":"Enter your Citibank Credit Card User ID and Password. Please note that if you have not accessed Citibank Account Online in the past 6 months, please register again and choose a new User ID.",
"address":{
"city":"New York",
"state":"NY",
"country":"USA",
"postalCode":"10043",
"addressLine1":"",
"addressLine2":""
},
"currency":"USD",
"email":"https://online.citibank.com/US/Welcome.c",
"oauthInstitutionId":null
}
]
}
'''


class TestInstitutionsListResponse(unittest.TestCase):

    def test_institutions_response(self):
        response_dict = json.loads(EXAMPLE_INSTITUTIONS_RESPONSE)
        response = InstitutionsListResponse.from_dict(response_dict)
        self.assertEqual({}, response._unused_fields)
        for institution in response.institutions:
            self.assertEqual({}, institution._unused_fields)
            self.assertEqual({}, institution.address._unused_fields)
